(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('setimmediate'), require('httpplease/plugins/json'), require('httpplease'), require('qs')) :
	typeof define === 'function' && define.amd ? define(['exports', 'setimmediate', 'httpplease/plugins/json', 'httpplease', 'qs'], factory) :
	(factory((global.StratumnAgentClient = {}),null,global.json,global.httpplease,global.qs));
}(this, (function (exports,setimmediate,json,httpplease,qs) { 'use strict';

json = json && json.hasOwnProperty('default') ? json['default'] : json;
httpplease = httpplease && httpplease.hasOwnProperty('default') ? httpplease['default'] : httpplease;

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function deprecated(oldFunc, newFunc) {
  if (!newFunc) {
    console.warn("WARNING: " + oldFunc + " is deprecated.");
  } else {
    console.warn("WARNING: " + oldFunc + " is deprecated. Please use " + newFunc + " instead.");
  }
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

/**
 * Calls a function that returns a Promise until a condition is reached
 * @param {function} condition - while condition is true body will keep being called
 * @param {function} body - a function that is repeatedly called while condition is true
 * @returns {Promise} a Promise that resolves when the condition is no longer true
 */
function promiseWhile(condition, body) {
  return new Promise(function (resolve, reject) {
    function loop() {
      body().then(function () {
        // When the result of calling `condition` is no longer true, we are
        // done.
        if (!condition()) {
          resolve();
        } else {
          loop();
        }
      }).catch(reject);
    }

    // Start running the loop in the next tick so that this function is
    // completely async. It would be unexpected if `body` was called
    // synchronously the first time.
    setImmediate(loop);
  });
}

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};





var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();





var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();









































var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

var DEFAULT_BATCH_SIZE = 20;

function findSegments(adaptor, process) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var options = Object.assign({}, opts);
  if (opts.limit === -1) {
    options.limit = options.batchSize || DEFAULT_BATCH_SIZE;
    delete options.batchSize;
    options.offset = 0;
    var lastBatch = [];
    var result = [];

    return promiseWhile(function () {
      return lastBatch.length === options.limit;
    }, function () {
      return findSegments(adaptor, process, options).then(function (newSegments) {
        lastBatch = newSegments;
        result.push.apply(result, toConsumableArray(newSegments));
        options.offset += options.limit;
      });
    }).then(function () {
      return result;
    });
  }
  return adaptor.findSegments(process.name, opts).then(function (res) {
    return res.body.map(function (obj) {
      return segmentify(adaptor, process, obj);
    });
  });
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function getBranches(adaptor, agent, prevLinkHash) {
  var tags = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

  deprecated('Agent#getBranches(agent, prevLinkHash, tags = [])', 'Agent#findSegments(agent, filter)');

  var opts = tags ? { prevLinkHash: prevLinkHash, tags: tags } : { prevLinkHash: prevLinkHash };

  return findSegments(adaptor, agent, opts);
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function segmentify(adaptor, process, obj) {
  Object.keys(process.processInfo.actions).filter(function (key) {
    return ['init'].indexOf(key) < 0;
  }).forEach(function (key) {
    /*eslint-disable*/
    obj[key] = function () {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return adaptor.createSegment.apply(adaptor, [process.name, obj.meta.linkHash, key].concat(args)).then(function (res) {
        return segmentify(adaptor, process, res.body);
      });
    };
  });

  /*eslint-disable*/
  obj.getPrev = function () {
    /*eslint-enable*/
    if (obj.link.meta.prevLinkHash) {
      return process.getSegment(obj.link.meta.prevLinkHash);
    }

    return Promise.resolve(null);
  };

  // Deprecated.
  /*eslint-disable*/
  obj.load = function () {
    /*eslint-enable*/
    deprecated('segment#load()');
    return Promise.resolve(segmentify(adaptor, process, {
      link: JSON.parse(JSON.stringify(obj.link)),
      meta: JSON.parse(JSON.stringify(obj.meta))
    }));
  };

  // Deprecated.
  /*eslint-disable*/
  obj.getBranches = function () {
    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    /*eslint-enable*/
    return getBranches.apply(undefined, [adaptor, process, obj.meta.linkHash].concat(args));
  };

  return obj;
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function createMap(adaptor, process) {
  for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  return adaptor.createMap.apply(adaptor, [process.name].concat(args)).then(function (res) {
    return segmentify(adaptor, process, res.body);
  });
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function getSegment(adaptor, process, linkHash) {
  return adaptor.getSegment(process.name, linkHash).then(function (res) {
    return segmentify(adaptor, process, res.body);
  });
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function getMapIds(adaptor, process) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  return adaptor.getMapIds(process.name, opts).then(function (res) {
    return res.body;
  });
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function getLink(adaptor, process, hash) {
  deprecated('Agent#getLink(agent, hash)', 'Agent#getSegment(agent, hash)');

  return getSegment(adaptor, process, hash);
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function getMap(adaptor, process, mapId) {
  var tags = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

  deprecated('getMap(agent, mapId, tags = [])', 'findSegments(agent, filter)');

  var opts = tags ? { mapIds: mapId, tags: tags } : { mapIds: mapId };
  return findSegments(adaptor, process, opts);
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

// Deprecated.
function processify(adaptor, process) {
  var updatedProcess = process;
  if (adaptor.url) {
    updatedProcess.agentUrl = adaptor.url;
    updatedProcess.prefixUrl = adaptor.url + '/' + process.name;
  }
  updatedProcess.createMap = createMap.bind(null, adaptor, updatedProcess);
  updatedProcess.getSegment = getSegment.bind(null, adaptor, updatedProcess);
  updatedProcess.findSegments = findSegments.bind(null, adaptor, updatedProcess);
  updatedProcess.getMapIds = getMapIds.bind(null, adaptor, updatedProcess);

  // Deprecated.
  updatedProcess.getBranches = getBranches.bind(null, adaptor, updatedProcess);
  updatedProcess.getLink = getLink.bind(null, adaptor, updatedProcess);
  updatedProcess.getMap = getMap.bind(null, adaptor, updatedProcess);

  return updatedProcess;
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function getProcesses(adaptor) {
  return adaptor.getProcesses().then(function (res) {
    return res.body.map(processify.bind(null, adaptor));
  });
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

var request = httpplease.use(json);

function send(method, url, args) {
  return new Promise(function (resolve, reject) {
    request({ method: method, url: url, body: args }, function (err, res) {
      if (err) {
        var error = err && err.body && err.body.meta && err.body.meta.errorMessage ? new Error(err.body.meta.errorMessage) : err;
        error.status = err.status;
        reject(error);
      } else {
        resolve(res);
      }
    });
  });
}

function get$1(url) {
  return send('GET', url);
}

function post(url, args) {
  return send('POST', url, args);
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/
/**
 * Makes a query string.
 * @param {object} obj - an object of keys
 * @returns {string} a query string
 */
function makeQueryString(obj) {
  // use brackets format for compatibility with GO:
  // https://github.com/google/go-querystring
  // see also conversation in:
  // https://github.com/stratumn/indigo-js/pull/18
  var query = qs.stringify(obj, { arrayFormat: 'brackets' });
  if (query.length) {
    return '?' + query;
  }
  return '';
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

var httpAdaptor = function () {
  function httpAdaptor(url) {
    classCallCheck(this, httpAdaptor);

    this.agentUrl = url;
  }

  createClass(httpAdaptor, [{
    key: 'getInfo',
    value: function getInfo() {
      return get$1(this.url);
    }
  }, {
    key: 'getProcesses',
    value: function getProcesses() {
      return get$1(this.url + '/processes');
    }
  }, {
    key: 'createMap',
    value: function createMap(processName) {
      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      return post(this.url + '/' + processName + '/segments', args);
    }
  }, {
    key: 'getSegment',
    value: function getSegment(processName, linkHash) {
      return get$1(this.url + '/' + processName + '/segments/' + linkHash);
    }
  }, {
    key: 'findSegments',
    value: function findSegments(processName) {
      var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      return get$1(this.url + '/' + processName + '/segments' + makeQueryString(opts));
    }
  }, {
    key: 'getMapIds',
    value: function getMapIds(processName) {
      var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      return get$1(this.url + '/' + processName + '/maps' + makeQueryString(opts));
    }
  }, {
    key: 'createSegment',
    value: function createSegment(processName, linkHash, action) {
      for (var _len2 = arguments.length, args = Array(_len2 > 3 ? _len2 - 3 : 0), _key2 = 3; _key2 < _len2; _key2++) {
        args[_key2 - 3] = arguments[_key2];
      }

      return post(this.url + '/' + processName + '/segments/' + linkHash + '/' + action, args);
    }
  }, {
    key: 'url',
    get: function get$$1() {
      return this.agentUrl;
    }
  }]);
  return httpAdaptor;
}();

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

// need to clone data as client will modify it!
var decorateBody = function decorateBody(res) {
  return { body: JSON.parse(JSON.stringify(res)) };
};

var objectAdaptor = function () {
  function objectAdaptor(agent) {
    classCallCheck(this, objectAdaptor);

    this.agent = agent;
  }

  createClass(objectAdaptor, [{
    key: "getInfo",
    value: function getInfo() {
      return this.agent.getInfo().then(decorateBody);
    }
  }, {
    key: "getProcesses",
    value: function getProcesses() {
      return Promise.resolve(decorateBody(this.agent.getAllProcesses()));
    }
  }, {
    key: "createMap",
    value: function createMap(processName) {
      try {
        var _agent$getProcess;

        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        return (_agent$getProcess = this.agent.getProcess(processName)).createMap.apply(_agent$getProcess, args).then(decorateBody);
      } catch (err) {
        return Promise.reject(err);
      }
    }
  }, {
    key: "getSegment",
    value: function getSegment(processName, linkHash) {
      try {
        return this.agent.getProcess(processName).getSegment(linkHash).then(decorateBody);
      } catch (err) {
        return Promise.reject(err);
      }
    }
  }, {
    key: "findSegments",
    value: function findSegments(processName) {
      var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      try {
        return this.agent.getProcess(processName).findSegments(opts).then(decorateBody);
      } catch (err) {
        return Promise.reject(err);
      }
    }
  }, {
    key: "getMapIds",
    value: function getMapIds(processName) {
      var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      try {
        return this.agent.getProcess(processName).getMapIds(opts).then(decorateBody);
      } catch (err) {
        return Promise.reject(err);
      }
    }
  }, {
    key: "createSegment",
    value: function createSegment(processName, linkHash, action) {
      try {
        var _agent$getProcess2;

        for (var _len2 = arguments.length, args = Array(_len2 > 3 ? _len2 - 3 : 0), _key2 = 3; _key2 < _len2; _key2++) {
          args[_key2 - 3] = arguments[_key2];
        }

        return (_agent$getProcess2 = this.agent.getProcess(processName)).createSegment.apply(_agent$getProcess2, [linkHash, action].concat(args)).then(decorateBody);
      } catch (err) {
        return Promise.reject(err);
      }
    }
  }, {
    key: "url",
    get: function get$$1() {
      return null;
    }
  }]);
  return objectAdaptor;
}();

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function getAdaptor(objectOrUrl) {
  if (typeof objectOrUrl === 'string') {
    return new httpAdaptor(objectOrUrl);
  } else if ((typeof objectOrUrl === 'undefined' ? 'undefined' : _typeof(objectOrUrl)) === 'object') {
    return new objectAdaptor(objectOrUrl);
  }

  throw new Error('The argument passed is neither a url or an object!');
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function getAgent(objectOrUrl) {
  try {
    var adaptor = getAdaptor(objectOrUrl);
    return adaptor.getInfo().then(function (res) {
      var agent = res.body;
      agent.url = adaptor.url;
      agent.getProcesses = getProcesses.bind(null, adaptor);
      agent.processes = Object.keys(agent.processes).map(function (key) {
        return agent.processes[key];
      }).reduce(function (map, p) {
        var updatedMap = map;
        updatedMap[p.name] = processify(adaptor, p);
        return updatedMap;
      }, {});
      return agent;
    });
  } catch (err) {
    return Promise.reject(err);
  }
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function fromSegment(obj) {
  return getAgent(obj.meta.agentUrl || obj.meta.applicationLocation).then(function (agent) {
    if (!agent.processes[obj.link.meta.process]) {
      throw new Error('process \'' + obj.link.meta.process + '\' not found');
    }
    var adaptor = getAdaptor(agent.url);
    var segment = segmentify(adaptor, agent.processes[obj.link.meta.process], obj);
    return { process: agent.processes[obj.link.meta.process], segment: segment };
  });
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

// Deprecated.
var config = {
  applicationUrl: 'https://%s.stratumn.rocks'
};

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function getApplication(name, url) {
  deprecated('getApplication(name, url)', 'getAgent(url)');

  return getAgent(url || config.applicationUrl.replace('%s', name));
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

function loadLink(adaptor, obj) {
  deprecated('loadLink(obj)', 'fromSegment(obj)');

  return fromSegment(adaptor, obj).then(function (_ref) {
    var segment = _ref.segment;
    return segment;
  });
}

/*
  Copyright 2017 Stratumn SAS. All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

exports.getAgent = getAgent;
exports.fromSegment = fromSegment;
exports.getApplication = getApplication;
exports.loadLink = loadLink;
exports.config = config;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=stratumn-agent-client.js.map
